# Generated by Django 2.1.2 on 2018-11-03 20:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djams', '0003_auto_20181023_2302'),
    ]

    operations = [
        migrations.AddField(
            model_name='adm_app',
            name='description',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='Description'),
        ),
    ]



# '''
# Tests of django_plotly_dash.

# These tests can be directly used or imported into eg the tests of the demo app.

# The use of pytest is assumed.

# Copyright (c) 2018 Gibbs Consulting and others - see CONTRIBUTIONS.md

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# '''

# import pytest

# #pylint: disable=bare-except

# def test_dash_app():
#     'Test the import and formation of the dash app orm wrappers'

#     from django_plotly_dash.models import StatelessApp
#     stateless_a = StatelessApp(app_name="Some name")

#     assert stateless_a
#     assert stateless_a.app_name
#     assert str(stateless_a) == stateless_a.app_name

# def test_util_error_cases(settings):
#     'Test handling of missing settings'

#     settings.PLOTLY_DASH = None

#     from django_plotly_dash.util import pipe_ws_endpoint_name, dpd_http_endpoint_root, http_endpoint, insert_demo_migrations

#     assert pipe_ws_endpoint_name() == 'dpd/ws/channel'
#     assert dpd_http_endpoint_root() == "dpd/views"
#     assert http_endpoint("fred") == '^dpd/views/fred/$'
#     assert not insert_demo_migrations()

#     del settings.PLOTLY_DASH

#     assert pipe_ws_endpoint_name() == 'dpd/ws/channel'
#     assert dpd_http_endpoint_root() == "dpd/views"
#     assert http_endpoint("fred") == '^dpd/views/fred/$'
#     assert not insert_demo_migrations()

# def test_demo_routing():
#     'Test configuration options for the demo'

#     from django_plotly_dash.util import pipe_ws_endpoint_name, insert_demo_migrations
#     assert pipe_ws_endpoint_name() == 'ws/channel'
#     assert insert_demo_migrations()

# @pytest.mark.django_db
# def test_direct_access(client):
#     'Check direct use of a stateless application using demo test data'

#     from django.urls import reverse
#     from .app_name import main_view_label

#     for route_name in ['layout', 'dependencies', main_view_label]:
#         for prefix, arg_map in [('app-', {'ident':'SimpleExample'}),
#                                 ('', {'ident':'simpleexample-1'}),]:
#             url = reverse('the_django_plotly_dash:%s%s' % (prefix, route_name), kwargs=arg_map)

#             response = client.get(url)

#             assert response.content
#             assert response.status_code == 200

#     for route_name in ['routes',]:
#         for prefix, arg_map in [('app-', {'ident':'SimpleExample'}),
#                                 ('', {'ident':'simpleexample-1'}),]:
#             url = reverse('the_django_plotly_dash:%s%s' % (prefix, route_name), kwargs=arg_map)

#             did_fail = False
#             try:
#                 response = client.get(url)
#             except:
#                 did_fail = True

#             assert did_fail

# @pytest.mark.django_db
# def test_updating(client):
#     'Check updating of an app using demo test data'

#     import json
#     from django.urls import reverse

#     route_name = 'update-component'

#     for prefix, arg_map in [('app-', {'ident':'SimpleExample'}),
#                             ('', {'ident':'simpleexample-1'}),]:
#         url = reverse('the_django_plotly_dash:%s%s' % (prefix, route_name), kwargs=arg_map)

#         response = client.post(url, json.dumps({'output':{'id':'output-size', 'property':'children'},
#                                                 'inputs':[{'id':'dropdown-color',
#                                                            'property':'value',
#                                                            'value':'blue'},
#                                                           {'id':'dropdown-size',
#                                                            'property':'value',
#                                                            'value':'medium'},
#                                                          ]}), content_type="application/json")

#         assert response.content == b'{"response": {"props": {"children": "The chosen T-shirt is a medium blue one."}}}'
#         assert response.status_code == 200

# @pytest.mark.django_db
# def test_injection_app_access(client):
#     'Check direct use of a stateless application using demo test data'

#     from django.urls import reverse
#     from .app_name import main_view_label

#     for route_name in ['layout', 'dependencies', main_view_label]:
#         for prefix, arg_map in [('app-', {'ident':'dash_example_1'}),
#                                 #('', {'ident':'simpleexample-1'}),
#                                ]:
#             url = reverse('the_django_plotly_dash:%s%s' % (prefix, route_name), kwargs=arg_map)

#             response = client.get(url)

#             assert response.content
#             assert response.status_code == 200

#     for route_name in ['routes',]:
#         for prefix, arg_map in [('app-', {'ident':'dash_example_1'}),]:
#             url = reverse('the_django_plotly_dash:%s%s' % (prefix, route_name), kwargs=arg_map)

#             did_fail = False
#             try:
#                 response = client.get(url)
#             except:
#                 did_fail = True

#             assert did_fail

# @pytest.mark.django_db
# def test_injection_updating(client):
#     'Check updating of an app using demo test data'

#     import json
#     from django.urls import reverse

#     route_name = 'update-component'

#     for prefix, arg_map in [('app-', {'ident':'dash_example_1'}),]:
#         url = reverse('the_django_plotly_dash:%s%s' % (prefix, route_name), kwargs=arg_map)

#         response = client.post(url, json.dumps({'output':{'id':'test-output-div', 'property':'children'},
#                                                 'inputs':[{'id':'my-dropdown1',
#                                                            'property':'value',
#                                                            'value':'TestIt'},
#                                                          ]}), content_type="application/json")

#         rStart = b'{"response": {"props": {"children":'

#         assert response.content[:len(rStart)] == rStart
#         assert response.status_code == 200

#         have_thrown = False

#         try:
#             client.post(url, json.dumps({'output':{'id':'test-output-div2', 'property':'children'},
#                                          'inputs':[{'id':'my-dropdown2',
#                                                     'property':'value',
#                                                     'value':'TestIt'},
#                                                   ]}), content_type="application/json")
#         except:
#             have_thrown = True

#         assert have_thrown

#         session = client.session
#         session['django_plotly_dash'] = {'django_to_dash_context': 'Test 789 content'}
#         session.save()

#         response3 = client.post(url, json.dumps({'output':{'id':'test-output-div2', 'property':'children'},
#                                                  'inputs':[{'id':'my-dropdown2',
#                                                             'property':'value',
#                                                             'value':'TestIt'},
#                                                           ]}), content_type="application/json")
#         rStart3 = b'{"response": {"props": {"children":'

#         assert response3.content[:len(rStart3)] == rStart3
#         assert response3.status_code == 200

#         assert response3.content.find(b'Test 789 content') > 0



# Generated by Django 2.1.1 on 2018-10-23 22:02

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djams', '0002_mds_partytype_partyclass'),
    ]

    operations = [
        migrations.RenameField(
            model_name='mds_taxtype',
            old_name='generic_tax_type_abbreviation',
            new_name='generic_tax_type_abbr',
        ),
    ]



# Generated by Django 2.1.1 on 2018-10-21 22:38

from django.conf import settings
import django.contrib.auth.models
import django.contrib.auth.validators
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sessions', '0001_initial'),
        ('auth', '0009_alter_user_last_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='ADM_CustomUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('username', models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.UnicodeUsernameValidator()], verbose_name='username')),
                ('first_name', models.CharField(blank=True, max_length=30, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=150, verbose_name='last name')),
                ('email', models.EmailField(blank=True, max_length=254, verbose_name='email address')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='ADM_App',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200, verbose_name='App Name')),
                ('django_app_name', models.CharField(error_messages={'unique': 'This django_app_name has already been taken.'}, max_length=150, null=True, unique=True, verbose_name='Django App Name')),
                ('version', models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='App Version')),
                ('user_launch_view', models.CharField(blank=True, max_length=200, null=True, verbose_name='Launch View')),
                ('launch_url', models.URLField(blank=True, null=True)),
                ('launch_local_url', models.URLField(blank=True, null=True)),
                ('launch_dev_url', models.URLField(blank=True, null=True)),
                ('launch_qa_url', models.URLField(blank=True, null=True)),
                ('launch_uat_url', models.URLField(blank=True, null=True)),
                ('launch_prod_url', models.URLField(blank=True, null=True)),
                ('support_start_date', models.DateField(blank=True, null=True, verbose_name='Support Start Date')),
                ('support_end_date', models.DateField(blank=True, null=True, verbose_name='Support End Date')),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('app_thumbnail', models.ImageField(upload_to='app_thumbnails/')),
            ],
        ),
        migrations.CreateModel(
            name='ADM_App_Data_Dest',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('folder_identifier', models.CharField(blank=True, max_length=200, null=True)),
                ('file_descriptor', models.CharField(blank=True, max_length=200, null=True)),
                ('relevant_model', models.CharField(max_length=200)),
                ('use_django_app_model_prefix', models.BooleanField(default=True)),
                ('custom_upload_query', models.CharField(blank=True, max_length=1000, null=True)),
                ('app', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='app_data_desc_to_app', to='djams.ADM_App')),
            ],
        ),
        migrations.CreateModel(
            name='ADM_AppCategory',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('keyword', models.CharField(blank=True, max_length=150, null=True)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='ADM_AppCategoryType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(max_length=200)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='appcategorytype_creator', to=settings.AUTH_USER_MODEL)),
                ('last_update_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='appcategorytype_updater', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='ADM_AppRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='approle_creator', to=settings.AUTH_USER_MODEL)),
                ('last_update_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='approle_updater', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='ADM_AppSession',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('token', models.CharField(blank=True, max_length=150, null=True)),
                ('username', models.CharField(blank=True, max_length=100, null=True)),
                ('app_name', models.CharField(blank=True, max_length=150, null=True)),
                ('event_time', models.DateTimeField(blank=True, null=True)),
                ('user_ip', models.CharField(blank=True, max_length=100, null=True)),
                ('app', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='appsession_app', to='djams.ADM_App')),
            ],
        ),
        migrations.CreateModel(
            name='ADM_AppSessionAudit',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('token', models.CharField(blank=True, max_length=150, null=True)),
                ('username', models.CharField(blank=True, max_length=100, null=True)),
                ('app_name', models.CharField(blank=True, max_length=150, null=True)),
                ('event_time', models.DateTimeField(blank=True, null=True)),
                ('event_result', models.CharField(blank=True, max_length=100, null=True)),
                ('operating_username', models.CharField(blank=True, max_length=150, null=True)),
                ('process', models.CharField(blank=True, max_length=100, null=True)),
                ('user_ip', models.CharField(blank=True, max_length=100, null=True)),
                ('activity_desc', models.CharField(blank=True, max_length=300, null=True)),
                ('app', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='appsessionaudit_app', to='djams.ADM_App')),
            ],
        ),
        migrations.CreateModel(
            name='ADM_Authority',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('regions_used', models.BooleanField(default=False)),
                ('groups_used', models.BooleanField(default=False)),
                ('valid_from_date', models.DateField(blank=True, null=True)),
                ('valid_to_date', models.DateField(blank=True, null=True)),
                ('current', models.BooleanField(default=True)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('app', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='authority_app', to='djams.ADM_App')),
            ],
            options={
                'ordering': ['client', 'user', 'app'],
            },
        ),
        migrations.CreateModel(
            name='ADM_DevTeam',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='devteam_creator', to=settings.AUTH_USER_MODEL)),
                ('deputy_team_lead', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='devteam_dep_lead_user', to=settings.AUTH_USER_MODEL)),
                ('last_update_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='devteam_updater', to=settings.AUTH_USER_MODEL)),
                ('team_lead', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='devteam_lead_user', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='ADM_EnquiryMessage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('message', models.TextField(max_length=2000, null=True)),
                ('message_read', models.BooleanField(default='False')),
                ('team_responded', models.BooleanField(default=False)),
                ('enquirer_responded', models.BooleanField(default=False)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='enquirymessage_creator', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['creation_date'],
            },
        ),
        migrations.CreateModel(
            name='ADM_EnquiryStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='enquiry_status_creator', to=settings.AUTH_USER_MODEL)),
                ('last_update_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='enquiry_status_updater', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='ADM_EventType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('description', models.CharField(max_length=200, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='ADM_HelpPage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('narrative', models.CharField(blank=True, max_length=4000, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='ADM_SelfServiceUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.CharField(max_length=200)),
                ('first_name', models.CharField(max_length=100)),
                ('last_name', models.CharField(max_length=100)),
                ('dev_account', models.CharField(max_length=100)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='selfserviceuser_creator', to=settings.AUTH_USER_MODEL)),
                ('dev_team', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='selfserviceuser_devteam', to='djams.ADM_DevTeam')),
                ('last_update_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='selfserviceuser_updater', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='ADM_Subscription',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('subscription_paid', models.BooleanField(default=False)),
                ('amount_paid', models.DecimalField(decimal_places=2, max_digits=38)),
                ('uses_regions', models.BooleanField(default=True)),
                ('uses_groups', models.BooleanField(default=True)),
                ('start_date', models.DateField(blank=True, null=True)),
                ('end_date', models.DateField(blank=True, null=True)),
                ('currently_subscribed', models.BooleanField(blank=True, null=True)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('app', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='subscription_solution', to='djams.ADM_App')),
            ],
            options={
                'ordering': ['app'],
            },
        ),
        migrations.CreateModel(
            name='ADM_SubscriptionType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('length', models.CharField(blank=True, choices=[('m', 'Monthly'), ('a', 'Annual'), ('c', 'Custom Fee')], max_length=2, null=True)),
                ('days_subscribed', models.IntegerField(blank=True, null=True)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='subscriptiontype_creator', to=settings.AUTH_USER_MODEL)),
                ('last_update_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='subscriptiontype_updater', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='ADM_Team',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='team_creator', to=settings.AUTH_USER_MODEL)),
                ('deputy_team_lead', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='dep_team_lead_user', to=settings.AUTH_USER_MODEL)),
                ('last_update_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='team_updater', to=settings.AUTH_USER_MODEL)),
                ('team_lead', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='team_lead_user', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='ADM_UserAppRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('app', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='userapprole_app', to='djams.ADM_App')),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='userapprole_creator', to=settings.AUTH_USER_MODEL)),
                ('last_update_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='userapprole_updater', to=settings.AUTH_USER_MODEL)),
                ('role', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='userapprole_role', to='djams.ADM_AppRole')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='userapprole_user', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['user', 'app', 'role'],
            },
        ),
        migrations.CreateModel(
            name='ADM_UserEnquiry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('reference', models.TextField(blank=True, max_length=50, null=True)),
                ('details', models.TextField(help_text='Enter your enquiry here', max_length=2000, null=True)),
                ('status_datetime_changed', models.DateTimeField(blank=True, null=True)),
                ('opened_date', models.DateField(auto_now=True)),
                ('closed_date', models.DateField(blank=True, null=True)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('name', models.CharField(max_length=200)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('app', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='userenquiry_app', to='djams.ADM_App')),
            ],
        ),
        migrations.CreateModel(
            name='ADM_UserProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('avatar', models.ImageField(blank=True, default='user_thumbnails/default.jpg', null=True, upload_to='')),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('require_password_reset', models.BooleanField(default='False')),
                ('invalid_login_attempts', models.IntegerField(blank=True, default=0, null=True)),
                ('password_change_date', models.DateField(blank=True, null=True)),
                ('pp1', models.TextField(blank=True, max_length=128, null=True)),
                ('ps1', models.TextField(blank=True, max_length=128, null=True)),
                ('pp2', models.TextField(blank=True, max_length=128, null=True)),
                ('ps2', models.TextField(blank=True, max_length=128, null=True)),
                ('pp3', models.TextField(blank=True, max_length=128, null=True)),
                ('ps3', models.TextField(blank=True, max_length=128, null=True)),
                ('pp4', models.TextField(blank=True, max_length=128, null=True)),
                ('ps4', models.TextField(blank=True, max_length=128, null=True)),
                ('pp5', models.TextField(blank=True, max_length=128, null=True)),
                ('ps5', models.TextField(blank=True, max_length=128, null=True)),
                ('pp6', models.TextField(blank=True, max_length=128, null=True)),
                ('ps6', models.TextField(blank=True, max_length=128, null=True)),
                ('pp7', models.TextField(blank=True, max_length=128, null=True)),
                ('ps7', models.TextField(blank=True, max_length=128, null=True)),
                ('pp8', models.TextField(blank=True, max_length=128, null=True)),
                ('ps8', models.TextField(blank=True, max_length=128, null=True)),
                ('pp9', models.TextField(blank=True, max_length=128, null=True)),
                ('ps9', models.TextField(blank=True, max_length=128, null=True)),
                ('pp10', models.TextField(blank=True, max_length=128, null=True)),
                ('ps10', models.TextField(blank=True, max_length=128, null=True)),
                ('pp11', models.TextField(blank=True, max_length=128, null=True)),
                ('ps11', models.TextField(blank=True, max_length=128, null=True)),
                ('pp12', models.TextField(blank=True, max_length=128, null=True)),
                ('ps12', models.TextField(blank=True, max_length=128, null=True)),
                ('pp13', models.TextField(blank=True, max_length=128, null=True)),
                ('ps13', models.TextField(blank=True, max_length=128, null=True)),
                ('pp14', models.TextField(blank=True, max_length=128, null=True)),
                ('ps14', models.TextField(blank=True, max_length=128, null=True)),
                ('pp15', models.TextField(blank=True, max_length=128, null=True)),
                ('ps15', models.TextField(blank=True, max_length=128, null=True)),
                ('pp16', models.TextField(blank=True, max_length=128, null=True)),
                ('ps16', models.TextField(blank=True, max_length=128, null=True)),
                ('pp17', models.TextField(blank=True, max_length=128, null=True)),
                ('ps17', models.TextField(blank=True, max_length=128, null=True)),
                ('pp18', models.TextField(blank=True, max_length=128, null=True)),
                ('ps18', models.TextField(blank=True, max_length=128, null=True)),
                ('pp19', models.TextField(blank=True, max_length=128, null=True)),
                ('ps19', models.TextField(blank=True, max_length=128, null=True)),
                ('pp20', models.TextField(blank=True, max_length=128, null=True)),
                ('ps20', models.TextField(blank=True, max_length=128, null=True)),
                ('pp21', models.TextField(blank=True, max_length=128, null=True)),
                ('ps21', models.TextField(blank=True, max_length=128, null=True)),
                ('pp22', models.TextField(blank=True, max_length=128, null=True)),
                ('ps22', models.TextField(blank=True, max_length=128, null=True)),
                ('pp23', models.TextField(blank=True, max_length=128, null=True)),
                ('ps23', models.TextField(blank=True, max_length=128, null=True)),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='user_creator', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='ADM_UserSession',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('session', models.ForeignKey(db_constraint=False, on_delete=django.db.models.deletion.CASCADE, to='sessions.Session')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='MDS_ClientGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150)),
                ('description', models.CharField(max_length=150)),
                ('valid_from_date', models.DateField(blank=True, null=True)),
                ('valid_to_date', models.DateField(blank=True, null=True)),
                ('current', models.BooleanField(default=True)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='MDS_ClientRegion',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150)),
                ('description', models.CharField(max_length=150)),
                ('valid_from_date', models.DateField(blank=True, null=True)),
                ('valid_to_date', models.DateField(blank=True, null=True)),
                ('current', models.BooleanField(default=True)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='MDS_Company',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('registered_name', models.CharField(max_length=200)),
                ('gfis_id', models.CharField(blank=True, max_length=50, null=True)),
                ('short_name', models.CharField(blank=True, max_length=40, null=True)),
                ('database_name', models.CharField(help_text='Enter the name of the client specific SQL database', max_length=200, unique=True)),
                ('logo', models.ImageField(blank=True, default='company_thumbnails/media.jpg', null=True, upload_to='company_thumbnails', verbose_name='Company Logo Thumbnail')),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='company_creator', to=settings.AUTH_USER_MODEL)),
                ('last_update_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='company_updater', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['registered_name'],
            },
        ),
        migrations.CreateModel(
            name='MDS_Country',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('capital_city', models.CharField(blank=True, max_length=200, null=True)),
                ('iso_numeric', models.IntegerField(blank=True, null=True)),
                ('iso_alpha2', models.CharField(max_length=2)),
                ('iso_alpha3', models.CharField(max_length=3)),
                ('iso_version', models.CharField(max_length=30, null=True)),
                ('valid_from_date', models.DateField(blank=True)),
                ('valid_to_date', models.DateField(blank=True, null=True)),
                ('latest_version', models.BooleanField(default=False)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='country_creator', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['id'],
            },
        ),
        migrations.CreateModel(
            name='MDS_CountryGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('abbreviation', models.CharField(blank=True, max_length=20, null=True)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='countrygroup_creator', to=settings.AUTH_USER_MODEL)),
                ('last_update_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='countrygroup_updater', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='MDS_Currency',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=75)),
                ('iso_alpha3', models.CharField(blank=True, max_length=3, null=True)),
                ('iso_numeric', models.IntegerField(blank=True, null=True)),
                ('symbol', models.CharField(blank=True, max_length=20, null=True)),
                ('minor_unit', models.IntegerField(blank=True, null=True)),
                ('valid_from_date', models.DateField(blank=True, null=True)),
                ('valid_to_date', models.DateField(blank=True, null=True)),
                ('latest_version', models.BooleanField(default='True')),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('country', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='currency_country', to='djams.MDS_Country')),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='currency_creator', to=settings.AUTH_USER_MODEL)),
                ('last_update_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='currency_updater', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='MDS_FinancialInstrumentType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('generic_instrument_type_name', models.CharField(max_length=200)),
                ('generic_instrument_type_abbreviation', models.CharField(blank=True, max_length=20, null=True)),
                ('instrument_region', models.CharField(choices=[('n', 'National'), ('c', 'Continental'), ('i', 'International')], max_length=200)),
                ('tax_category', models.CharField(max_length=200, null=True)),
                ('region_specific_instrument_type_name', models.CharField(max_length=200, null=True)),
                ('specific_instrument_type_abbr', models.CharField(blank=True, max_length=20, null=True)),
                ('start_date', models.DateField(blank=True, null=True)),
                ('end_date', models.DateField(blank=True, null=True)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='fininsttype_creator', to=settings.AUTH_USER_MODEL)),
                ('jurisdiction', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djams.MDS_Country')),
                ('last_update_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='fininsttype_updater', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='MDS_PartyClass',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(max_length=50)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='partyclass_creator', to=settings.AUTH_USER_MODEL)),
                ('last_update_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='partyclass_updater', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='MDS_PartyType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(max_length=100)),
                ('abbreviation', models.CharField(max_length=20)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('country', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='partytype_country', to='djams.MDS_Country')),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='partytype_creator', to=settings.AUTH_USER_MODEL)),
                ('last_update_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='partytype_updater', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='MDS_PublicBody',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150)),
                ('description', models.CharField(blank=True, max_length=300, null=True)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='MDS_PublicBodyType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150)),
                ('description', models.CharField(max_length=300)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='publicbodytype_creator', to=settings.AUTH_USER_MODEL)),
                ('last_update_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='publicbodytype_updater', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='MDS_TaxType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('generic_tax_type_name', models.CharField(max_length=200)),
                ('generic_tax_type_abbreviation', models.CharField(blank=True, max_length=10, null=True)),
                ('tax_region', models.CharField(choices=[('l', 'Local Government Tax'), ('s', 'State Tax'), ('n', 'National/Federal Tax')], max_length=200)),
                ('tax_category', models.CharField(max_length=200, null=True)),
                ('region_specific_tax_type_name', models.CharField(max_length=200, null=True)),
                ('specific_tax_type_abbr', models.CharField(blank=True, max_length=10, null=True)),
                ('start_date', models.DateField(blank=True, null=True)),
                ('end_date', models.DateField(blank=True, null=True)),
                ('creation_date', models.DateField(auto_now_add=True)),
                ('last_update_date', models.DateField(auto_now=True)),
                ('creation_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='taxtype_creator', to=settings.AUTH_USER_MODEL)),
                ('jurisdiction', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='djams.MDS_Country')),
                ('last_update_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='taxtype_updater', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='RDS_Company_Data_Ingest_Meta',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file_name', models.CharField(blank=True, max_length=255, null=True)),
                ('upload_user', models.CharField(blank=True, max_length=100, null=True)),
                ('upload_username', models.CharField(blank=True, max_length=100, null=True)),
                ('client', models.CharField(blank=True, max_length=200, null=True)),
                ('django_app_name', models.CharField(blank=True, max_length=150, null=True)),
                ('event_datetime', models.DateTimeField(blank=True, null=True)),
                ('event_result', models.CharField(blank=True, max_length=100, null=True)),
                ('activity_desc', models.CharField(blank=True, max_length=300, null=True)),
            ],
        ),
        migrations.AddField(
            model_name='mds_publicbody',
            name='body_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='publicbody_publicbodytype', to='djams.MDS_PublicBodyType'),
        ),
        migrations.AddField(
            model_name='mds_publicbody',
            name='country',
            field=models.ManyToManyField(related_name='publicbody_country', to='djams.MDS_Country'),
        ),
        migrations.AddField(
            model_name='mds_publicbody',
            name='creation_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='publicbody_creator', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='mds_publicbody',
            name='last_update_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='publicbody_updater', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='mds_country',
            name='group_membership',
            field=models.ManyToManyField(to='djams.MDS_CountryGroup'),
        ),
        migrations.AddField(
            model_name='mds_country',
            name='last_update_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='country_updater', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='mds_company',
            name='party_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='party_party_type', to='djams.MDS_PartyType'),
        ),
        migrations.AddField(
            model_name='mds_company',
            name='registered_country',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='party_country', to='djams.MDS_Country'),
        ),
        migrations.AddField(
            model_name='mds_clientregion',
            name='client',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='clientregion_company', to='djams.MDS_Company'),
        ),
        migrations.AddField(
            model_name='mds_clientregion',
            name='countries',
            field=models.ManyToManyField(to='djams.MDS_Country'),
        ),
        migrations.AddField(
            model_name='mds_clientregion',
            name='creation_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='clientregion_creator', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='mds_clientregion',
            name='last_update_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='clientregion_updater', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='mds_clientgroup',
            name='client',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='clientgroup_company', to='djams.MDS_Company'),
        ),
        migrations.AddField(
            model_name='mds_clientgroup',
            name='countries',
            field=models.ManyToManyField(to='djams.MDS_Country'),
        ),
        migrations.AddField(
            model_name='mds_clientgroup',
            name='creation_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='clientgroup_creator', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='mds_clientgroup',
            name='last_update_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='clientgroup_updater', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_userprofile',
            name='employer',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='user_employer', to='djams.MDS_Company'),
        ),
        migrations.AddField(
            model_name='adm_userprofile',
            name='employerco',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='employer_company', to='djams.MDS_Company'),
        ),
        migrations.AddField(
            model_name='adm_userprofile',
            name='last_update_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='user_updater', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_userprofile',
            name='resident_country',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='user_country', to='djams.MDS_Country'),
        ),
        migrations.AddField(
            model_name='adm_userprofile',
            name='user',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_userenquiry',
            name='client',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='userenquiry_company', to='djams.MDS_Company'),
        ),
        migrations.AddField(
            model_name='adm_userenquiry',
            name='creation_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='userenquiry_creator', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_userenquiry',
            name='last_update_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='userenquiry_status_updater', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_userenquiry',
            name='status',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='userenquiry_enquirystatus', to='djams.ADM_EnquiryStatus'),
        ),
        migrations.AddField(
            model_name='adm_userenquiry',
            name='team',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='userenquiry_team', to='djams.ADM_Team'),
        ),
        migrations.AddField(
            model_name='adm_userenquiry',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='userenquiry_user', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_subscription',
            name='client',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='subscription_party', to='djams.MDS_Company'),
        ),
        migrations.AddField(
            model_name='adm_subscription',
            name='creation_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='subscription_creator', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_subscription',
            name='last_update_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='subscription_updater', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_subscription',
            name='subscription_paid_currency',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='subscription_currency', to='djams.MDS_Currency'),
        ),
        migrations.AddField(
            model_name='adm_subscription',
            name='subscription_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='substype_subs', to='djams.ADM_SubscriptionType'),
        ),
        migrations.AddField(
            model_name='adm_selfserviceuser',
            name='team',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='selfserviceuser_team', to='djams.ADM_Team'),
        ),
        migrations.AddField(
            model_name='adm_selfserviceuser',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='selfserviceuser_user', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_enquirymessage',
            name='enquiry_thread',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='enquirymessage_userenquiry', to='djams.ADM_UserEnquiry'),
        ),
        migrations.AddField(
            model_name='adm_enquirymessage',
            name='last_update_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='enquirymessage_updater', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_enquirymessage',
            name='message_author',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='enquirymessage_user', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_enquirymessage',
            name='team',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='enquirymessage_team', to='djams.ADM_Team'),
        ),
        migrations.AddField(
            model_name='adm_authority',
            name='client',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='authority_company', to='djams.MDS_Company'),
        ),
        migrations.AddField(
            model_name='adm_authority',
            name='client_group',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='authority_clientgroup', to='djams.MDS_ClientGroup'),
        ),
        migrations.AddField(
            model_name='adm_authority',
            name='client_region',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='authority_clientregion', to='djams.MDS_ClientRegion'),
        ),
        migrations.AddField(
            model_name='adm_authority',
            name='creation_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='authority_creator', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_authority',
            name='last_update_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='authority_updater', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_authority',
            name='subscription',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='authority_subscription', to='djams.ADM_Subscription'),
        ),
        migrations.AddField(
            model_name='adm_authority',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='authority_user', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_appsessionaudit',
            name='client',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='appsessionaudit_client', to='djams.MDS_Company'),
        ),
        migrations.AddField(
            model_name='adm_appsessionaudit',
            name='operating_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='appsessionaudit_user', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_appsessionaudit',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='appsession_user_audit', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_appsession',
            name='client',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='appsession_client', to='djams.MDS_Company'),
        ),
        migrations.AddField(
            model_name='adm_appsession',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='appsession_user', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_appcategory',
            name='app_category_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='appcategory_type', to='djams.ADM_AppCategoryType'),
        ),
        migrations.AddField(
            model_name='adm_appcategory',
            name='creation_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='appcategory_creator', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_appcategory',
            name='last_update_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='appcategory_updater', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_app',
            name='app_category',
            field=models.ManyToManyField(blank=True, to='djams.ADM_AppCategory', verbose_name='App Categories'),
        ),
        migrations.AddField(
            model_name='adm_app',
            name='billing_recipient',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='billing_recipient', to=settings.AUTH_USER_MODEL, verbose_name='Billing Recipient'),
        ),
        migrations.AddField(
            model_name='adm_app',
            name='creation_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='app_creator', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_app',
            name='finance_sme',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='finance_sme', to=settings.AUTH_USER_MODEL, verbose_name='Finance SME'),
        ),
        migrations.AddField(
            model_name='adm_app',
            name='instrument_category',
            field=models.ManyToManyField(blank=True, to='djams.MDS_FinancialInstrumentType', verbose_name='Financial Instrument Category'),
        ),
        migrations.AddField(
            model_name='adm_app',
            name='last_update_user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='app_updater', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='adm_app',
            name='sales_contact',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='sales_contact', to=settings.AUTH_USER_MODEL, verbose_name='Sales Contact'),
        ),
        migrations.AddField(
            model_name='adm_app',
            name='tax_sme',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='tax_sme', to=settings.AUTH_USER_MODEL, verbose_name='Tax SME'),
        ),
        migrations.AddField(
            model_name='adm_app',
            name='tax_type',
            field=models.ManyToManyField(blank=True, to='djams.MDS_TaxType', verbose_name='Tax Type'),
        ),
        migrations.AddField(
            model_name='adm_app',
            name='tech_expert',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='tech_solution_expert', to=settings.AUTH_USER_MODEL, verbose_name='Technology Solution Expert'),
        ),
    ]



# Generated by Django 2.1.2 on 2018-11-04 16:01

import django.core.files.storage
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djams', '0006_auto_20181104_0115'),
    ]

    operations = [
        migrations.AlterField(
            model_name='adm_app',
            name='app_thumbnail',
            field=models.ImageField(storage=django.core.files.storage.FileSystemStorage(location='C:\\Users\\eburm\\Documents\\MEGA\\platform_code\\project_management\\../media'), upload_to='app_thumbnails'),
        ),
        migrations.AlterField(
            model_name='adm_app',
            name='description',
            field=models.CharField(blank=True, max_length=350, null=True, verbose_name='Description'),
        ),
    ]



'''
The bindings for analytical microsites
'''
# pylint: disable=wrong-import-position,wrong-import-order
from django.urls import path, include, re_path
from django.conf.urls import url
from django.contrib.auth import views as auth_views
from django.views.generic import TemplateView
from . import views

app_name = 'djams'

urlpatterns = [
                # path('', views.launch_app, name='launch-app'),
                re_path(r'login/$', views.signin, name='login'),
                path('appsession', views.appsession, name='appsession'),

                url(r'^userlogout/$', views.logout_redirect, name='userlogout'),
                path('logout/', views.endappsession, name='logout'),                
                path('endappsession', views.endappsession, name='endappsession'),

                url(r'^change-password/$', views.change_password, name='change-password'),
                path('passwordchanged/', views.passwordchanged, name='passwordchanged'),

                path('enquiries/', views.enquirylist, name='enquiries'),
                path('myenquiries/', views.enquirylist, name='myenquiries'),
                path('newenquiry', views.newenquiry, name='new-enquiry'),
                path('status_userenquiry/<pk>', views.status_userenquiry, name='status-userenquiry'),

                path('noaccess', TemplateView.as_view(template_name='djams/noaccess.html'), name='noaccess'),
                path('site-privacy-notice', views.enquirylist, name='site-privacy-notice'),
                path('djams-home-page', views.djams_home_page, name='djams-home-page'),

                path('my-clients/', views.my_clients, name='my-clients'),
                path('client/<pk>/', views.client_select, name='client-select'),
          

]


'''
App configuration information
'''

from django.apps import AppConfig


class DjamsConfig(AppConfig):
    'Verbose name and other settings for the django-plotly-dash application'
    name = 'djams'
    verbose_name = "Django Analytics Microsites"



# Generated by Django 2.1.2 on 2018-11-05 23:12

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djams', '0007_auto_20181104_1601'),
    ]

    operations = [
        migrations.AddField(
            model_name='adm_team',
            name='team_members',
            field=models.ManyToManyField(related_name='team_members_users', to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 2.1.2 on 2018-11-04 01:13

import django.core.files.storage
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djams', '0004_adm_app_description'),
    ]

    operations = [
        migrations.AlterField(
            model_name='adm_app',
            name='app_thumbnail',
            field=models.ImageField(storage=django.core.files.storage.FileSystemStorage(location='C:\\Users\\eburm\\Documents\\MEGA\\platform_code\\admin_portal\\../media\\app_thumbnails'), upload_to=''),
        ),
    ]



# Generated by Django 2.1.1 on 2018-10-23 21:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djams', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='mds_partytype',
            name='partyclass',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='partytype_partyclass', to='djams.MDS_PartyClass'),
        ),
    ]



import datetime
from django.core.paginator import Paginator
from .models import ADM_AppSessionAudit, ADM_App
from .config_vars import APP_PORTALS


def refineFilterQuery(queryDict):
    '''
    Description: This is used to refine Django view filters in order to remove parameter values such as 'ALL' or blank values, which would suggest that the user wants all available info
    Inputs: queryDict - a dictionary containing parameter keys and the user set values from dropdowns.
    Returns: a filtered dictionary which can be used in a call to a database object 
    '''
    # filter out "ALL" values:
    filtered = {}
    queryDict = {i:queryDict[i] for i in queryDict if 'ALL' not in queryDict[i]}
    
    # filter out blank values in lists:
    for i in queryDict:
        value = queryDict[i]
        if isinstance(value, list) != True and 'All' not in value and value != '':
            filtered[i] = value
        elif isinstance(queryDict[i], list):
            if '' not in value:
                filtered[i] = value
    
    return filtered

def obtainForeignKeyObjects(queryDict, model):
    '''
    Description:
    Inputs: queryDict - a django models query dictionary.
            model - a standard Django model with a foreign key
    Returns: a dictionary containing the object versions of the foreign keys to be filtered on
    N.B. on models: This funciton assumes that the primary key of the foreign kety models renamed 'id
    '''

    for field in model._meta.fields:
        if field.get_internal_type() == 'ForeignKey' and field.name in list(queryDict.keys()):
            print("Found a foreign key. Getting object: " + field.name)
            temp = {}
            temp['id'] = queryDict[field.name]
            print(str(temp))
            foreign_model = model._meta.get_field(field.name).remote_field.model
            print("Foreign model is: %s" %(str(foreign_model)))
            foreign_object_list = foreign_model.objects.filter(**temp)
            if len(foreign_object_list) > 0:
                queryDict[field.name] = foreign_object_list[0]
    
    return queryDict

def getSetAppInfo(request):
    '''
    Description: Sets the current app name in the request.session object 
    '''
    main_app = request.request_apps[0]
    main_app_name = main_app.split('.')[0]
    print("The main app name is: %s" %(main_app_name))

    # SERVER_ENV = settings.SERVER_ENV

    if main_app_name in APP_PORTALS:
        request.session['myappname'] = main_app_name

    else:
        sessionapp = ADM_App.objects.get(django_app_name=main_app_name)
        print("getSetAppInfo: current session app is: %s" %(str(sessionapp)))
        request.session['myapp'] = sessionapp.id
        request.session['myappname'] = sessionapp.name

    return

def paginate_pages(request, page_list, page_entries=10):

    page = request.GET.get('page', 1)
    paginator = Paginator(page_list, page_entries)
    try:
        messages = paginator.page(page)
    except PageNotAnInteger:
        messages = paginator.page(1)
    except EmptyPage:
        messages = paginator.page(paginator.num_pages)
    
    return page, paginator, messages

def update_creation_fields(form_inst, user):
    form_inst.creation_user = user
    form_inst.creation_date = datetime.datetime.today().strftime("%Y-%m-%d")
    return form_inst

def update_last_update_fields(form_inst, user):
    form_inst.last_update_user = user
    form_inst.last_update_date = datetime.datetime.today().strftime("%Y-%m-%d")
    return form_inst

def auditlog(request, viewname, action, result, username='', subjectuser='', subjectusername='', client=None):
    '''
    Description: Used for auditing user actions such as microsite logging in and logging out
    '''

    auditrecord = ADM_AppSessionAudit.objects.create()
    try:
        auditrecord.token = request.session['appsession']
    except:
        auditrecord.token = None

    try:
        auditrecord.user = request.user
    except:
        auditrecord.user = None
    
    if username is not "":
        auditrecord.username = username

    else:
        auditrecord.username = request.user.username

    auditrecord.client = client

    request_apps = request.request_apps

    getSetAppInfo(request)
    auditrecord.appname = request.session['myappname']

    auditrecord.app = None
    auditrecord.eventitme = datetime.datetime.now()

    auditrecord.eventresult = result
    if subjectuser is not None:
        auditrecord.subjectuser = subjectuser
    auditrecord.subjectusername = subjectusername
    auditrecord.process = viewname
    auditrecord.userip = '0.0.0.0'
    x_forwarded_for = request.META.get('REMOTE_ADDR')
    auditrecord.description = action
    auditrecord.save()
    print("Audit record saved.")

    return ('ADM_AppSessionAudit audit log written')


from django.shortcuts import render, get_object_or_404, redirect
from django.contrib.auth.decorators import login_required, user_passes_test
from django.contrib.auth import login, authenticate, update_session_auth_hash, logout
from django.core.paginator import Paginator
from django.db.models import Max
from django.contrib import messages
from django.urls import reverse_lazy
from django.core.exceptions import ObjectDoesNotExist
from django.conf import settings
import datetime
from datetime import timedelta
import dateutil.parser
import hashlib
from django.http import HttpResponseRedirect, HttpResponse


from .models import *
from .djams_forms import *
from .config_vars import SUPERUSER_APPS, APP_PORTALS
# from .djams_model_views import *

def reset_to_baseurl(request):
    return HttpResponseRedirect('')

@login_required
def launch_app(request):
    '''
    Description: this is the intitla view used when the app is first launched
    '''
    return redirect('djams:appsession')

def djams_home_page(request):
    '''
    Description: This is a link to the main home page of the app portal
    '''
    return redirect('https://www.burmicz.com')


def djams_privacy_notice(request):
    '''
    Description: Privacy notice for the app portal
    '''
    return render(request, 'djams/djams_privacy_notice.html')

def getSetAppInfo(request):
    '''
    Description: Sets the current app name in the request.session object 
    '''
    main_app = request.request_apps[0]
    main_app_name = main_app.split('.')[0]
    print("The main app name is: %s" %(main_app_name))

    # SERVER_ENV = settings.SERVER_ENV

    if main_app_name in APP_PORTALS:
        request.session['myappname'] = main_app_name

    else:
        sessionapp = ADM_App.objects.get(django_app_name=main_app_name)
        print("getSetAppInfo: current session app is: %s" %(str(sessionapp)))
        request.session['myapp'] = sessionapp.id
        request.session['myappname'] = sessionapp.name

    return

def getCurrentClientInfo(request, context):
    '''
    Description: Gets the name of the current client using the session context in request.
        Returns: A "context" dict. This will either contain info on the current client or nothing if no client has been selected 
    '''

    try:
        myclient = MDS_Company.objects.get(id=request.sesssion['myclient'])
        context['myclient'] = myclient.registered_name
        context['myclientid'] = myclient.id
        return context
    except KeyError as ke:
        print("getCurrentClientInfo: No client currently selected.")
        return context

def getCurrentAppInfo(request, context):
    '''
    Description: Gets the name of the app in the microsite using the session context in request
        Returns: A "context" dictionary. This is left untouched if no app selected.
    '''
    try:
        getSetAppInfo(request)
        myapp = request.session['myapp']
        myapprecord = ADM_App.objects.get(id=myapp)
        context['myapp'] = myapprecord.name
        context['myappid'] = myapprecord.id
        return context
    except KeyError as ke:
        print("getCurrentAppInfo: No app currently selected.")
        return context


def getAllClients(request, context):
    '''
    Description: Gets a list of clients using the session context in request
        Returns: a context dictionary with a list of clients or it is left untouched if no clients available to use user
    '''

    try:
        context['clients'] = UserClients_View.objects.filter(user_id=request.user.id).filter(app_id=request.session['myapp'])
        return context
    except KeyError as ke:
        print("getAllClients: No app currently selected or no clients available to view")
        return context

def getUserAppClientInfo(request):
    '''
    Description: Gets info relating to the current App and Client combination for use in a template. E.g. it can be used fopr populating a sidebar with the name of the current app and client in use 
                 for the current session
    '''
    context = {}
    context = getAllClients(request, context)
    context = getCurrentClientInfo(request, context)
    context = getCurrentAppInfo(request, context)
    context['userid'] = request.user.id
    return context


@login_required
def my_clients(request):
    '''
    Description: This function shows a list of clients available to the user. This is required before retrieving any solid data for your app since the data will be client specific.
    '''

    context = getUserAppClientInfo(request)
    context = getAllClients(request, context)
    main_app_name = request.request_apps[0].split('.')[0]

    return render(request, 'djams/myclients.html', context=context)


@login_required
def client_select(request):
    '''
    Description: This function is necessary for multi-client apps requiring database separation at the client level. This function is called after the user has selected from a list of clients available to them.
                It then sets the avaialble sub client databases in the request.session object, which is used for routing object requests
                '''

    # retrieve the current session record from the database:
    sessionuser = request.user.id
    appsession = ADM_AppSession.objects.get(user_id=sessionuser)
    sessiontoken = appsession.token
    sessiontime = dateime.dateime.now()
    sessionclient = pk

    # get the app id based on the current apps django_app_name:
    main_app_name = settings.AUTHORIZED_APPS[0].split('.')[0]
    current_app = ADM_App.objects.get(django_app_name=main_app_name)

    appsession.client_id = sessionclient
    appsesssion.logintime = sessiontime
    appsession.save()

    # copy the session data to the session audit log:
    sessionaudit = ADM_AppSessionAudit.objects.create(token=appsession.token, user_id=request.user.id, client_id=sessionclient, app_id=current_app.id, eventtime=sessiontime)
    sessionaudit.save()

    # set session vartiables for the app to make iti easy for app developer to get basic information
    company = MDS_Company.objects.get(id=sessionclient)
    current_database = str(company.database)

    request.session['myclient'] = sessionclient
    request.session['myclientname'] = MDS_Company.registered_name
    request.session['myapp'] = current_app.id
    request.session['myappname'] = current_app.name
    request.session['rds'] = "RDS_" + current_database
    request.session['tds'] = "TDS_" + current_database
    request.session['ads'] = "ADS_" + current_database

    dash_context = request.session.get('django_plotly_dash', dict())
    dash_context['rds'] = request.session['rds']
    dash_context['tds'] = request.session['tds']
    dash_context['ads'] = request.session['ads']
    request.session['django_plotly_dash'] = dash_context

    print("Printing client specific info:\n%s\n%s\n%s\n%s\n%s" %(request.session['myclientname'], request.session['myappname'], request.session['rds'], request.session['tds'], request.session['ads']))    

    try:
        myrolerecord = ADM_UserAppRole.objects.get(user=request.user.id, app=sessionapp.id)
        request.session['myrole'] = myrolerecord.role.name
    except:
        request.session['myrole'] = "Level 1 App User"
     
    return redirect('%s:app-home-page' %(main_app_name))

def auditlog(request, viewname, action, result, username='', subjectuser='', subjectusername='', client=None):
    '''
    Description: Used for auditing user actions such as microsite logging in and logging out
    '''

    auditrecord = ADM_AppSessionAudit.objects.create()
    try:
        auditrecord.token = request.session['appsession']
    except:
        auditrecord.token = None

    try:
        auditrecord.user = request.user
    except:
        auditrecord.user = None
    
    if username is not "":
        auditrecord.username = username

    else:
        auditrecord.username = request.user.username

    auditrecord.client = client

    request_apps = request.request_apps

    getSetAppInfo(request)
    auditrecord.appname = request.session['myappname']

    auditrecord.app = None
    auditrecord.eventitme = datetime.datetime.now()

    auditrecord.eventresult = result
    if subjectuser is not None:
        auditrecord.subjectuser = subjectuser
    auditrecord.subjectusername = subjectusername
    auditrecord.process = viewname
    auditrecord.userip = '0.0.0.0'
    x_forwarded_for = request.META.get('REMOTE_ADDR')
    auditrecord.description = action
    auditrecord.save()
    print("Audit record saved.")

    return ('ADM_AppSessionAudit audit log written')

def signUserIn(request, user, username):

    '''
    Description: Signing the user in if initial authentication was successful
    '''

    print("signUserIn: Logging user in")
    
    request.session['user_request_page'] = request.path

    login(request, user)
    user.adm_userprofile.invalid_login_attempts = 0
    user.save()

    if not request.session.exists(request.session.session_key):
        request.session.create() 

    # create user session
    instance = ADM_UserSession.objects.create(user=request.user, session_id=request.session.session_key)
    instance.save()
    # write audit record
    action = 'User successfully logged in to %s' %(str(request.request_apps))
    result = "Success"
    print("signUserIn: create audit log entry")
    auditlog(request, viewname="signin", action=action, result=result, username=request.user.username,
             subjectuser=None, subjectusername='', client=None)

    # find and deete any sessions using the same user's credentials
    usersessions = ADM_UserSession.objects.filter(user_id=request.user.id).exclude(session_id=request.session.session_key).values()
    for session in usersessions:
        ustodelete = ADM_UserSession.objects.filter(session_id=session['session_id'])
        ustodelete.delete()
        roguesession = Session.objects.filter(session_key=session['session_id'])
        roguesession.delete()
    print("signUserin: Sign in complete")
    return

def dontSignUserIn(request, user, username):
    '''
    Description: If authentication was unsuccessful, this function is called and logs the failure
    '''

    auditrecord = ADM_AppSessionAudit.objects.create()
    action = "Unsuccessful attempt to log in to %s" %(str(request.request_apps))
    result = 'Fail'
    auditlog(request, "signin", action, result, username, None, '')
    try:
        asserteduser = ADM_CustomUserobjects.filter(username=username).first()
        loginattempt = int(asserteduser.adm_userprofile.invalid_login_attempts) + 1
        print("dontSignUserIn: Login attempts so far: %s" %(loginattempts))
        asserteduser.adm_userprofile.invalid_login_attempts = loginattempts
        if loginattempts > 5:
            print("dontSignUserIn: loginattempts > 5. Setting user as no longer active")
            asserteduser.is_active = 0
        asserteduser.save()
    except:
        pass
    return

def apps_require_superuser_check(request_apps):
    '''
    Description: Checks the list of current apps being used as part of the app against the list in the SUPERUSERAPPS variable
    '''
    for request_app in request_apps:
        if any(super_app in request_app for super_app in SUPERUSER_APPS):
            return True
    return False

def signin(request):
    '''
    Description: The main custom signin function for authenticating users. The 'signUserIn' or 'dontSignUserin' functions are called \
                 depending on whether the user has been authenticated or not. All actions are recorded via the auditlog function 
    '''

    request_apps = request.request_apps
    main_request_app = request_apps[0].split('.')[0]

    if request.method == 'POST':
        form = LoginForm(request.POST)

        username = request.POST['username']
        print(username)

        password = request.POST['password']
        #print(password)
        user = authenticate(request, username=username, password=password)
        print("signIn: User authenticated object: %s" %(user))
        print("signIn: request_apps: %s" %(str(request_apps)))

        if request.use_microsite_auth:
            superuser_required = apps_require_superuser_check(request_apps)
        
        # elif request.admin:
        #     print("setting superuser_required (reason: request.admin)")
        #     superuser_required = True
        else:
            superuser_required = False
     
        if superuser_required:
        
            print("signin: This site requires superuser privelages. Checking if user holds priveleges")
            if user is not None:
                if user.is_staff is True:
                    print("signin: User is member of staff. Running 'signUserIn.")
                    signUserIn(request, user, username)
                    return redirect('djams:appsession')
                else:
                    print("User field is None")
                    return redirect('djams:noaccess')
            
            else:
                print("signin: User is not member of staff. Running 'dontSignUserIn'")
                dontSignUserIn(request, user, username)
                return redirect('djams:noaccess')
        
        else:
            print("signin: The apps used for this microsite do not require superuser privelages. ")

            if user is not None:
                print("signin: user field is not blank. starting SignUserIn.")
                signUserIn(request, user, username)
                return redirect('djams:appsession')
            
            else:
                print("signin: User field was received as 'None'. Not signing user in.")
                dontSignUserIn(request, user, username)
                return redirect('djams:login')
    
    else:
        print("signin: Producing a fresh user login form.")
        form = LoginForm(request.POST)

    return render(request, 'djams/login.html', {'form':form, 'main_request_app':main_request_app})

@login_required
def appsession(request):
    '''
    Description: Creates objects of AppSession denoting the start and end of an app session
    '''
    print("appsession. Setting session info.")
    sessionuser = request.user.id
    sessiontoken = str(uuid.uuid4())
    request.session['appsession'] = str(sessiontoken)
    sessiontime = datetime.datetime.now()
    main_app_name = request.request_apps[0].split('.')[0]

    # create an app session for the platform user:

    action = deleteAppSessionObj(request, sessionuser)

    newSession = ADM_AppSession.objects.create(user_id=sessionuser)
    newSession.save()

    result = 'Success'
    
    auditlog(request, viewname="appsession", action=action, result=result, username=request.user.username,
             subjectuser=None, subjectusername='', client=None)

    # copy the session data to the session audit log to store the app portals session cookie
    currentdate = dateutil.parser.parse(str(datetime.datetime.now())).date()
    asserteduser = ADM_CustomUser.objects.get(id=sessionuser)

    print("appsession: Running password history checks..")

    if asserteduser.adm_userprofile.password_change_date is None:
        print("appsession: User %s has never changed password" %(asserteduser))

        if asserteduser.is_staff == 1:
            days_upper_limit = 44
        else:
            days_upper_limit = 85

        asserteduser.adm_userprofile.password_change_date = currentdate - timedelta(days=days_upper_limit)
        print("appsession: password age: %s" %(asserteduser.adm_userprofile.password_change_date))
        asserteduser.save()

    days_since_pw_change = (currentdate - asserteduser.adm_userprofile.password_change_date).days

    if asserteduser.is_staff == 1:
        print("appsession: User is a staff member")

        if 40 < days_since_pw_change <= 45:
            return redirect('djams:change-password')
        
        elif days_since_pw_change > 45:
            asserteduser.is_active = 0
            asserteduser.save()
            return redirect('djams:logout')
    
    else:
        print("appsession: User is not a staff member")

        if 85 <= days_since_pw_change <= 90:
            return redirect('djams:change-password')
        
        elif days_since_pw_change > 90:
            asserteduser.is_active = 0
            asserteduser.save()
            return redirect('djams:logout')
        else:
            
            
            next_url = request.POST.get('next')
            print(request.POST)
            print(request.GET)

            if next_url:
                print("redirecting to next: %s" %(next_url))
                return HttpResponseRedirect(next_url)
            
            user_request_url = request.session.get('user_request_page', False)

            if user_request_url:
                user_request_url = str(user_request_url).replace('login/', '')
                print("user_Request_url: %s" %(user_request_url))
                return redirect(user_request_url)

            return redirect(main_app_name+':app-home-page')
    
    next_url = request.POST.get('next')
    if next_url:
        print("redirecting to next: %s" %(next_url))
        return HttpResponseRedirect(next_url)
    
    user_request_url = request.session.get('user_request_page', False)

    if user_request_url:
        user_request_url = str(user_request_url).replace('login/', '')
        print("user_Request_url: %s" %(user_request_url))
        return redirect(user_request_url)

    return redirect(main_app_name+':app-home-page')

def deleteAppSessionObj(request, sessionuser):

    if ADM_AppSession.objects.filter(user_id=sessionuser).exists():
        ADM_AppSession.objects.filter(user_id=sessionuser).delete()
        action = 'Old session token deleted and new one created'
    else:
        action = 'No old session to delete. New one created'

    return action

def logout_redirect(request):

    logout(request)

    return redirect('/')


@login_required
def endappsession(request):
    '''
    Description: Ends an app session
    '''
    sessionuser = request.user.id
    sessionusername = request.user.username
    sessiontoken = str(uuid.uuid4())
    sessiontime = datetime.datetime
    application_name = request.request_apps[0].split('.')[0]
    #request.session['app_redirect_name'] = application_name
    print("endappsession: Application name: %s" %(application_name))

    sessionprocess = 'Logout / End App Session'
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        sessionuserip = x_forwarded_for.split(',')[0]
    else:
        sessionuserip = request.META.get('REMOTE_ADDR')

    usersessions = ADM_UserSession.objects.filter(user_id=request.user.id).values()
    for session in usersessions:
        ustodelete = ADM_UserSession.objects.filter(session_id=session['session_id'])
        ustodelete.delete()


    action = deleteAppSessionObj(request, sessionuser)

    newSession = ADM_AppSession.objects.create(user_id=sessionuser)
    newSession.save()

    result = 'Success'
    
    auditlog(request, viewname="endappsession", action=action, result=result, username=request.user.username,
             subjectuser=None, subjectusername='', client=None)

    # return redirect('/')
    return redirect('djams:userlogout')

def check_redirect_page(request):

    BAN_REDIRECT_POST_LOGIN = ['login', 'change_password']

    for i in BAN_REDIRECT_POST_LOGIN:
        if i in request.session['user_request_page']:
            request.session['user_request_page'] = '/'
    return


@login_required
def change_password(request):

    print("user requested page: %s" %(str(request.session['user_request_page'])))

    if request.method == 'POST':
        form = PasswordChangeForm(request.user, request.POST)
        newpassword = request.POST['new_password1']
        asserteduser = ADM_CustomUser.objects.get(id=request.user.id)
        pp1 = asserteduser.adm_userprofile.pp1
        ps1 = asserteduser.adm_userprofile.ps1
        pp2 = asserteduser.adm_userprofile.pp2
        ps2 = asserteduser.adm_userprofile.ps2
        pp3 = asserteduser.adm_userprofile.pp3
        ps3 = asserteduser.adm_userprofile.ps3
        ####### ADDITION OTHER 20 PASSWORDS #####
        pp4 = asserteduser.adm_userprofile.pp4
        ps4 = asserteduser.adm_userprofile.ps4
        pp5 = asserteduser.adm_userprofile.pp5
        ps5 = asserteduser.adm_userprofile.ps5
        pp6 = asserteduser.adm_userprofile.pp6
        ps6 = asserteduser.adm_userprofile.ps6
        pp7 = asserteduser.adm_userprofile.pp7
        ps7 = asserteduser.adm_userprofile.ps7
        pp8 = asserteduser.adm_userprofile.pp8
        ps8 = asserteduser.adm_userprofile.ps8
        pp9 = asserteduser.adm_userprofile.pp9
        ps9 = asserteduser.adm_userprofile.ps9
        pp10 = asserteduser.adm_userprofile.pp10
        ps10 = asserteduser.adm_userprofile.ps10
        pp11 = asserteduser.adm_userprofile.pp11
        ps11 = asserteduser.adm_userprofile.ps11
        pp12 = asserteduser.adm_userprofile.pp12
        ps12 = asserteduser.adm_userprofile.ps12
        pp13 = asserteduser.adm_userprofile.pp13
        ps13 = asserteduser.adm_userprofile.ps13
        pp14 = asserteduser.adm_userprofile.pp14
        ps14 = asserteduser.adm_userprofile.ps14
        pp15 = asserteduser.adm_userprofile.pp15
        ps15 = asserteduser.adm_userprofile.ps15
        pp16 = asserteduser.adm_userprofile.pp16
        ps16 = asserteduser.adm_userprofile.ps16
        pp17 = asserteduser.adm_userprofile.pp17
        ps17 = asserteduser.adm_userprofile.ps17
        pp18 = asserteduser.adm_userprofile.pp18
        ps18 = asserteduser.adm_userprofile.ps18
        pp19 = asserteduser.adm_userprofile.pp19
        ps19 = asserteduser.adm_userprofile.ps19
        pp20 = asserteduser.adm_userprofile.pp20
        ps20 = asserteduser.adm_userprofile.ps20
        pp21 = asserteduser.adm_userprofile.pp21
        ps21 = asserteduser.adm_userprofile.ps21
        pp22 = asserteduser.adm_userprofile.pp22
        ps22 = asserteduser.adm_userprofile.ps22
        pp23 = asserteduser.adm_userprofile.pp23
        ps23 = asserteduser.adm_userprofile.ps23
        ###### END OF ADDITION 20 MORE PASSWORDS ####


        if pp1 is not None and ps1 is not None:
            hashpass1 = hashlib.sha256(ps1.encode() + newpassword.encode()).hexdigest() + ':' + ps1
        if pp2 is not None and ps2 is not None:
            hashpass2 = hashlib.sha256(ps2.encode() + newpassword.encode()).hexdigest() + ':' + ps2
        if pp3 is not None and ps3 is not None:
            hashpass3 = hashlib.sha256(ps3.encode() + newpassword.encode()).hexdigest() + ':' + ps3
        ####### ADDITION OTHER 20 PASSWORDS ######
        if pp4 is not None and ps4 is not None:
            hashpass4 = hashlib.sha256(ps4.encode() + newpassword.encode()).hexdigest() + ':' + ps4
        if pp5 is not None and ps5 is not None:
            hashpass5 = hashlib.sha256(ps5.encode() + newpassword.encode()).hexdigest() + ':' + ps5
        if pp6 is not None and ps6 is not None:
            hashpass6 = hashlib.sha256(ps6.encode() + newpassword.encode()).hexdigest() + ':' + ps6
        if pp7 is not None and ps7 is not None:
            hashpass7 = hashlib.sha256(ps7.encode() + newpassword.encode()).hexdigest() + ':' + ps7
        if pp8 is not None and ps8 is not None:
            hashpass8 = hashlib.sha256(ps8.encode() + newpassword.encode()).hexdigest() + ':' + ps8
        if pp9 is not None and ps9 is not None:
            hashpass9 = hashlib.sha256(ps9.encode() + newpassword.encode()).hexdigest() + ':' + ps9
        if pp10 is not None and ps10 is not None:
            hashpass10 = hashlib.sha256(ps10.encode() + newpassword.encode()).hexdigest() + ':' + ps10
        if pp11 is not None and ps11 is not None:
            hashpass11 = hashlib.sha256(ps11.encode() + newpassword.encode()).hexdigest() + ':' + ps11
        if pp12 is not None and ps12 is not None:
            hashpass12 = hashlib.sha256(ps12.encode() + newpassword.encode()).hexdigest() + ':' + ps12
        if pp13 is not None and ps13 is not None:
            hashpass13 = hashlib.sha256(ps13.encode() + newpassword.encode()).hexdigest() + ':' + ps13
        if pp14 is not None and ps14 is not None:
            hashpass14 = hashlib.sha256(ps14.encode() + newpassword.encode()).hexdigest() + ':' + ps14
        if pp15 is not None and ps14 is not None:
            hashpass15 = hashlib.sha256(ps15.encode() + newpassword.encode()).hexdigest() + ':' + ps15
        if pp16 is not None and ps16 is not None:
            hashpass16 = hashlib.sha256(ps16.encode() + newpassword.encode()).hexdigest() + ':' + ps16
        if pp17 is not None and ps17 is not None:
            hashpass17 = hashlib.sha256(ps17.encode() + newpassword.encode()).hexdigest() + ':' + ps17
        if pp18 is not None and ps18 is not None:
            hashpass18 = hashlib.sha256(ps18.encode() + newpassword.encode()).hexdigest() + ':' + ps18
        if pp19 is not None and ps19 is not None:
            hashpass19 = hashlib.sha256(ps19.encode() + newpassword.encode()).hexdigest() + ':' + ps19
        if pp20 is not None and ps20 is not None:
            hashpass20 = hashlib.sha256(ps20.encode() + newpassword.encode()).hexdigest() + ':' + ps20
        if pp21 is not None and ps21 is not None:
            hashpass21 = hashlib.sha256(ps21.encode() + newpassword.encode()).hexdigest() + ':' + ps21
        if pp22 is not None and ps22 is not None:
            hashpass22 = hashlib.sha256(ps22.encode() + newpassword.encode()).hexdigest() + ':' + ps22
        if pp23 is not None and ps23 is not None:
            hashpass23 = hashlib.sha256(ps23.encode() + newpassword.encode()).hexdigest() + ':' + ps23
        ###### END OF ADDITION 20 MORE PASSWORDS ####

        duplicatepass = False

        if pp1 is None:
            pass
        elif pp1 == hashpass1:
            duplicatepass = True
        if pp2 is None:
            pass
        elif pp2 == hashpass2:
            duplicatepass = True
        if pp3 is None:
            pass
        elif pp3 == hashpass3:
            duplicatepass = True
        ####### ADDITION OTHER 20 PASSWORDS #####
        if pp4 is None:
            pass
        elif pp4 == hashpass4:
            duplicatepass = True
        if pp5 is None:
            pass
        elif pp5 == hashpass5:
            duplicatepass = True
        if pp6 is None:
            pass
        elif pp6 == hashpass6:
            duplicatepass = True
        if pp7 is None:
            pass
        elif pp7 == hashpass7:
            duplicatepass = True
        if pp8 is None:
            pass
        elif pp8 == hashpass8:
            duplicatepass = True
        if pp9 is None:
            pass
        elif pp9 == hashpass9:
            duplicatepass = True
        if pp10 is None:
            pass
        elif pp10 == hashpass10:
            duplicatepass = True
        if pp11 is None:
            pass
        elif pp11 == hashpass11:
            duplicatepass = True
        if pp12 is None:
            pass
        elif pp12 == hashpass12:
            duplicatepass = True
        if pp13 is None:
            pass
        elif pp13 == hashpass13:
            duplicatepass = True
        if pp14 is None:
            pass
        elif pp14 == hashpass14:
            duplicatepass = True
        if pp15 is None:
            pass
        elif pp15 == hashpass15:
            duplicatepass = True
        if pp16 is None:
            pass
        elif pp16 == hashpass16:
            duplicatepass = True
        if pp17 is None:
            pass
        elif pp17 == hashpass17:
            duplicatepass = True
        if pp18 is None:
            pass
        elif pp18 == hashpass18:
            duplicatepass = True
        if pp19 is None:
            pass
        elif pp19 == hashpass19:
            duplicatepass = True
        if pp20 is None:
            pass
        elif pp20 == hashpass20:
            duplicatepass = True
        if pp21 is None:
            pass
        elif pp21 == hashpass21:
            duplicatepass = True
        if pp22 is None:
            pass
        elif pp22 == hashpass22:
            duplicatepass = True
        if pp23 is None:
            pass
        elif pp23 == hashpass23:
            duplicatepass = True
        ###### END OF ADDITION 20 OTHER PASSWORDS #####


        if newpassword == request.POST['old_password']:
            duplicatepass = True
        if duplicatepass == True:
            form.add_error('new_password1', "Password must be different to current password and past 23 passwords")
        if not any(x.isupper() for x in newpassword):
            form.add_error('new_password1', "Passwords must contain at least one capital letter")
        if not any(x.islower() for x in newpassword):
            form.add_error('new_password1', "Passwords must contain at least one lower case letter")
        if not any(x.isdigit() for x in newpassword):
            form.add_error('new_password1', "Passwords must contain at least one digit")
        if len(newpassword) < 10:
            form.add_error('new_password1', "Passwords must contain at least ten characters")

        if form.is_valid():
            user = form.save()
            user.adm_userprofile.password_change_date = datetime.datetime.now()
            pp1 = user.adm_userprofile.pp1
            ps1 = user.adm_userprofile.ps1
            pp2 = user.adm_userprofile.pp2
            ps2 = user.adm_userprofile.ps2
            pp3 = user.adm_userprofile.pp3
            ps3 = user.adm_userprofile.ps3
            #### ADD 20 PASSWORDS ####
            pp4 = user.adm_userprofile.pp4
            ps4 = user.adm_userprofile.ps4
            pp5 = user.adm_userprofile.pp5
            ps5 = user.adm_userprofile.ps5
            ps6 = user.adm_userprofile.ps6
            pp6 = user.adm_userprofile.pp6
            ps7 = user.adm_userprofile.ps7
            pp7 = user.adm_userprofile.pp7
            ps8 = user.adm_userprofile.ps8
            pp8 = user.adm_userprofile.pp8
            ps9 = user.adm_userprofile.ps9
            pp9 = user.adm_userprofile.pp9
            ps10 = user.adm_userprofile.ps10
            pp10 = user.adm_userprofile.pp10
            ps11 = user.adm_userprofile.ps11
            pp11 = user.adm_userprofile.pp11
            ps12 = user.adm_userprofile.ps12
            pp12 = user.adm_userprofile.pp12
            ps13 = user.adm_userprofile.ps13
            pp13 = user.adm_userprofile.pp13
            pp14 = user.adm_userprofile.pp14
            ps14 = user.adm_userprofile.ps14
            pp15 = user.adm_userprofile.pp15
            ps15 = user.adm_userprofile.ps15
            pp16 = user.adm_userprofile.pp16
            ps16 = user.adm_userprofile.ps16
            pp17 = user.adm_userprofile.pp17
            ps17 = user.adm_userprofile.ps17
            pp18 = user.adm_userprofile.pp18
            ps18 = user.adm_userprofile.ps18
            pp19 = user.adm_userprofile.pp19
            ps19 = user.adm_userprofile.ps19
            pp20 = user.adm_userprofile.pp20
            ps20 = user.adm_userprofile.ps20
            pp21 = user.adm_userprofile.pp21
            ps21 = user.adm_userprofile.ps21
            pp22 = user.adm_userprofile.pp22
            ps22 = user.adm_userprofile.ps22
            pp23 = user.adm_userprofile.pp23
            ps23 = user.adm_userprofile.ps23
            ### END OF ADD 20 PASSWORDS ###

            salt = uuid.uuid4().hex
            password = request.POST['old_password']
            user.adm_userprofile.pp1 = hashlib.sha256(salt.encode() + password.encode()).hexdigest() + ':' + salt
            user.adm_userprofile.ps1 = salt
            user.adm_userprofile.pp2 = pp1
            user.adm_userprofile.ps2 = ps1
            user.adm_userprofile.pp3 = pp2
            user.adm_userprofile.ps3 = ps2
            ### ADD 20 NEW PASSWORDS ###
            user.adm_userprofile.pp4 = pp3
            user.adm_userprofile.ps4 = ps3
            user.adm_userprofile.pp5 = pp4
            user.adm_userprofile.ps5 = ps4
            user.adm_userprofile.pp6 = pp5
            user.adm_userprofile.ps6 = ps5
            user.adm_userprofile.pp7 = pp6
            user.adm_userprofile.ps7 = ps6
            user.adm_userprofile.pp8 = pp7
            user.adm_userprofile.ps8 = ps7
            user.adm_userprofile.pp9 = pp8
            user.adm_userprofile.ps9 = ps8
            user.adm_userprofile.pp10 = pp9
            user.adm_userprofile.ps10 = ps9
            user.adm_userprofile.pp11 = pp10
            user.adm_userprofile.ps11 = ps10
            user.adm_userprofile.pp12 = pp11
            user.adm_userprofile.ps12 = ps11
            user.adm_userprofile.pp13 = pp12
            user.adm_userprofile.ps13 = ps12
            user.adm_userprofile.pp14 = pp13
            user.adm_userprofile.ps14 = ps13
            user.adm_userprofile.pp15 = pp14
            user.adm_userprofile.ps15 = ps14
            user.adm_userprofile.pp16 = pp15
            user.adm_userprofile.ps16 = ps15
            user.adm_userprofile.pp17 = pp16
            user.adm_userprofile.ps17 = ps16
            user.adm_userprofile.pp18 = pp17
            user.adm_userprofile.ps18 = ps17
            user.adm_userprofile.pp19 = pp18
            user.adm_userprofile.ps19 = ps18
            user.adm_userprofile.pp20 = pp19
            user.adm_userprofile.ps20 = ps19
            user.adm_userprofile.pp21 = pp20
            user.adm_userprofile.ps21 = ps20
            user.adm_userprofile.pp22 = pp21
            user.adm_userprofile.ps22 = ps21
            user.adm_userprofile.pp23 = pp22
            user.adm_userprofile.ps23 = ps22
            ### END OF 20 NEW PASSWORDS ###

            user.save()
            update_session_auth_hash(request, user)
            messages.success(request, 'Your password has been updated')
            action = "User's password successfully updated"
            result = "Success"
            auditlog(request, "ChangePassword", action, result, request.user.username, request.user,
                     request.user.username)

            #print("Redirecting: %s" %(request.session['user_request_page']))

            next_url = request.GET.get('next')
            if next_url:
                return HttpResponseRedirect(next_url)

            try:
                return redirect(request.session['user_request_page'])
            except KeyError as ke:
                return redirect(main_app_name+':app-home-page')
            #return redirect(request.session['user_request_page'])
        else:
            action = "User unsuccessfully attempted to update password"
            result = "Fail"
            auditlog(request, "change_password", action, result, request.user.username, request.user,
                     request.user.username)
            messages.error(request, 'Please correct the error')
    else:
        form = PasswordChangeForm(request.user)
    return render(request, 'djams/change_password.html', {'form': form})

def passwordchanged(request):
    return render(request, 'djams/passwordchanged.html')

@login_required
def enquirylist(request):

    request_apps = request.request_apps

    if 'admin_portal' in request_apps:
        print("enquirylist: App is admin_portal. Showing all enquiries.")
        enquiry_list = ADM_UserEnquiry.objects.filter(user_id=request.user.id).exclude(status__name='Closed').order_by('-opened_date')
    
    else:
        print("enquirylist. Showing user specific enquiries")
        enquiry_list = ADM_UserEnquiry.objects.filter(user_id=request.user.id).exclude(status__name='Closed').order_by('-opened_date')

    page = request.GET.get('page', 1)

    paginator = Paginator(enquiry_list, 10)

    try:
        enquiries = paginator.page(page)
    except PageNotAnInteger:
        enquiries = paginator.page(1)
    except EmptyPage:
        enquiries = paginator.page(paginator.num_pages)
    
    return render(request, 'djams/enquiries.html', {'enquiries': enquiries})


def newenquiry(request):
    '''
    Description: Allows logged in and anonymnous users to submit an enquiry
    '''
    
    page_context = {}
    
    try:
        refnum = ADM_UserEnquiry.objects.all().aggregate(Max('id'))['id__max']
    except:
        print("newenquiry: Exception")
        refnum = 0
    try:
        refnum += 1
    except TypeError as te:
        refnum = 1

    refstring = 'AP' + str(refnum)

    print(str(request.POST))

    if request.method == 'POST' and 'send' in request.POST:
        print('newenquiry: receiving data')
        enquiryform = UserEnquiryForm(request.POST)
        print("newenquiry: Form valid: %s" %(str(enquiryform.is_valid())))
        if enquiryform.is_valid():
            enquiry_inst = enquiryform.save(commit=False)
            enquiry_inst.user = request.user
            enquiry_inst.reference = refstring
            enquiry_inst.email = request.user.email
            enquiry_inst.opened_date = datetime.datetime.today().strftime("%Y-%m-%d")
            enquiry_inst.statuschanged = datetime.datetime.today()
            enquiry_inst.creation_user = request.user
            enquiry_inst.creation_date = datetime.datetime.today().strftime('%Y-%m-%d')
            enquiry_inst.last_update_date = datetime.datetime.today().strftime('%Y-%m-%d')
            enquiry_inst.last_update_user = request.user
            enquiry_inst.save()
            action = "User created new enquiry with reference %s with ID %s"  %(str(enquiry_inst.reference), enquiry_inst.id)
            result = 'Success'
            auditlog(request, viewname="newenquiry", action=action, result=result, username=request.user.username,
             subjectuser=None, subjectusername='', client=None)
            return redirect('djams:enquiries')
    
    else:
        page_context['enquiryform'] = UserEnquiryForm()
        page_context['additional_page_context'] = {'user': request.user,
                                                    'reference': refstring,
                                                    'email': request.user.email}
    return render(request, 'djams/new_userenquiry.html', context=page_context)


def paginate_pages(request, page_list, page_entries=10):

    page = request.GET.get('page', 1)
    paginator = Paginator(page_list, page_entries)
    try:
        messages = paginator.page(page)
    except PageNotAnInteger:
        messages = paginator.page(1)
    except EmptyPage:
        messages = paginator.page(paginator.num_pages)
    
    return page, paginator, messages

@login_required
def status_userenquiry(request, pk):
    '''
    Description: Logged in users can view the statuses of their enquiries
    '''
    page_context = {}

    # get the record details from the database using the primary key
    userenquiry_inst = get_object_or_404(UserEnquiry, pk=pk)
    page_context['userenquiry_inst'] = userenquiry_inst
    message_list = ADM_EnquiryMessage.objects.filter(enquiry=pk).order_by('-creation_date')
    mpage_context['additional_page_context'] = {'user': request.user,
                                                'reference': 'AP' + str(userenquiry_inst.id),
                                                'email': request.user.email,
                                                }

    # Paginations
    page, paginator, messages = paginate_pages(request, page_list=message_list, page_entries=5)


    if request.method == 'POST':
        print("status_userenquiry: receiving data")
        messageform = UserEnquiryMsgForm(request.POST, prefix='msg', instance=EnquiryMessage())
        print("status_userenquiry: form is valid: %s" %(messageform.is_valid()))

        if messageform.is_valid():
            for message in message_list:
                if message.message_read == False:
                    currentmessage = ADM_EnquiryMessage.objects.get(pk=message.id)
                    currentmessage.message_read = True
                    currentmessage.save()
        message_inst = messageform.save(commit=False)
        message_inst.user = request.user
        message_inst.message_read = False
        message_inst.creation_user = request.user
        message_inst.last_update_date = datetime.datetime.today().strftime("%Y-%m-%d")
        message_inst.last_update_user = request.user
        message_inst.save()

        return redirect('djams:status-userenquiry', pk=userenquiry_inst.id)

    else:
        page_context['enquiryform'] = UserEnquiryForm(instance=userenquiry_inst)
        page_contewxt['messageform'] = UserEnquiryMsgForm(prefix='msg', instance=EnquiryMessage())

    page_context['messages'] = messages

    return render(request, 'djams/status_userenquiry.html', context=page_context)
        


from django.core.exceptions import PermissionDenied
from .models import ADM_Authority, ADM_App
from django.conf import settings
from django.shortcuts import redirect
from django.http import HttpResponse


def login_app_required(function, redirect_field_name=None, login_url='login/'):
    
    '''
    Description: Checks users are logged in and subscribed to the current app
    '''

    def wrap(request, *args, **kwargs):

        # check auth:
        if not request.user.is_authenticated:
            request.session['user_request_page'] = request.path
            return redirect(login_url)
        
        app_name = request.resolver_match.app_name

        print("app_name: %s" %(app_name))

        # get the microsite / app name and check in the session whether they have already been authorized:
        # main_app_name = settings.DJAMS_AUTHORIZED_APPS[0].split('.')[0]
        user_auth_dict = request.session.get('user_app_auths', {})
        user_app_auth = user_auth_dict.get(app_name, False)
        print("Previous user app auth: %s" % (str(user_app_auth)))

        if user_app_auth:  # if previously authorized in the session, return the view
            print("login_app_required: :User previously permitted in session to use app.")
            return function(request, *args, **kwargs)
        
        else: # else perform checks that involve transactions to the database: 
            current_app = ADM_App.objects.get(django_app_name=app_name)
            app_authority = ADM_Authority.objects.filter(user=request.user, app=current_app).first()
            # print("user: %s" %(request.user.id))
            # print("current_app: %s" %(current_app.id))
            # print("Authority: %s" %(str(app_authority)))

            if app_authority:
                print("login_app_required: setting user_app_auth for current app as authenticated.")
                user_auth_dict[app_name] = True
                request.session['user_app_auths'] = user_auth_dict
                return function(request, *args, **kwargs)
            else:
                message = 'You need to be subscribed to an app in order to gain access to this page. Please contact the adminstrator / owner of the app you wish to view.'
                return HttpResponse(message, status=401)


    wrap.__doc__ = function.__doc__
    wrap.__name__ = function.__name__

    return wrap


def login_app_client_required(function, redirect_field_name=None, login_url=None):
    '''
    Description: Checks users are logged in and subscribed to the current app
    '''

    def wrap(request, *args, **kwargs):

        # check auth:
        if not request.user.is_authenticated:
            return redirect(login_url)

        # get the microsite / app name and check in the session whether they have already been authorized:
        app_name = request.resolver_match.app_name
        client = request.session.get('myclient', False)

        user_app_auth = request.session.get('user_app_client_auth', {}).get(app_name, False)
        # print("user_app_auth: %s" % (str(user_app_auth)))

        if user_app_auth:  # if previously authorized in the session, return the view
            print("login_app_client_required: :User previously permitted in session to use app and client combination.")
            return function(request, *args, **kwargs)

        else:  # else perform checks that involve db transactions:
            current_app = App.objects.get(django_app_name=app_name)
            app_authority = Authority.objects.filter(
                user=request.user, app=current_app).first()
            
            # print("user: %s" % (request.user.id))
            # print("current_app: %s" % (current_app.id))
            # print("Authority: %s" % (str(authority)))

            if app_authority and client:
                print("login_app_required: setting user_app_auth for current app as authenticated.")
                request.session.get('user_app_auth', {})[app_name] = True
                return function(request, *args, **kwargs)

            elif app_authority and not client:
                return redirect('etca_bindings:my-clients')

            else:
                raise PermissionDenied

    wrap.__doc__ = function.__doc__
    wrap.__name__ = function.__name__

    return wrap



''' 
These are the middleware bindings for routing authentication, access to master data and client specific database queries 
'''

import threading
from django.conf import settings

MASTER_DATASTORE = 'MDS'
MASTER_ADMIN_DATASTORE = 'MDS'
MASTER_ADMIN_APP_LABELS = ['auth', 'accounts', 'sessions', 'view_authorities', 'view_user_clients', 'djams', 'admin_portal', 'app_portal']
MASTER_ADMIN_MODEL_LABELS = ['statelessapp', ]
SUB_CLIENT_MODEL_FLAGS = ['rds_', 'tds_', 'ads_']
SUB_CLIENT_DB_FLAGS = ['RDS_', 'TDS_', 'ADS_']

# object to hold request data
request_cfg = threading.local()

class BaseMiddleware:
    '''
    Description: The base middleware, which determines the main microsite application being used, determined by the AUTHORIZED_APPS variable in the settings.py file of the main app
                 that is requesting access. The name of this app is then used for authenticaiton and audit purposes by the djams app.
    '''

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        from django.conf import settings
        request.request_apps = settings.DJAMS_AUTHORIZED_APPS
        request.use_microsite_auth = settings.DJAMS_MICROSITE_AUTHENTICATION
        response = self.get_response(request)
        return response

class DBRouterMiddleware(object):
    '''
    Description: This middlwares functions include:
                1. Sets a flag if we are acccessing Django admin in order to prevent database rerouting for the auth model. It then removes the flag once the request is processed
                2. Sets and uses the relevant client specific databases on a threaded request object
    '''

    def __init__(self, get_response):
        self.get_response = get_response
    
    def __call__(self, request):
        return self.get_response(request)

    def process_view(self, request, view_func, args, kwargs):
        
        if request.path.startswith('/admin'):
            request_cfg.admin = True
        

        try: # else raise error if not logged in
            if request.session['rds']:
                request_cfg.cfg_rds = request.session['rds']
                request_cfg.cfg_tds = request.session['tds']
                request_cfg.cfg_ads = request.sessions['ads']

        except KeyError as ke:
            pass
        except AttributeError as ke:
            pass


    def process_response(self, request, response):

        if hasattr(request_cfg, 'admin'):
            del request_cfg.admin

        if request_cfg['rds']:
            request_cfg.cfg_rds = request.session['rds']
            request_cfg.cfg_tds = request.session['tds']
            request_cfg.cfg_ads = request.session['ads']

    def process_request(self, request, response):
        
        if request_cfg['rds']:
            request_cfg.cfg_rds = request.session['rds']
            request_cfg.cfg_tds = request.session['tds']
            request_cfg.cfg_ads = request.session['ads']

class UserSessionRouter(object):

    '''
    Description: Redirects database IO for the Django auth and sessions. This maintains authentication between microsite apps
    '''

    def db_for_read(self, model, **hints):

        if not hasattr(request_cfg, 'admin'):

            if model._meta.app_label in MASTER_ADMIN_APP_LABELS:
                return MASTER_ADMIN_DATASTORE

            elif model.__name__.lower() in MASTER_ADMIN_MODEL_LABELS:
                return MASTER_ADMIN_DATASTORE

        
        return None

    def db_for_write(self, model, **hints):

        if not hasattr(request_cfg, 'admin'):
            if model._meta.app_label in MASTER_ADMIN_APP_LABELS:
                return MASTER_ADMIN_DATASTORE

            elif model.__name__.lower() in MASTER_ADMIN_MODEL_LABELS:
                return MASTER_ADMIN_DATASTORE

        return None

    def allow_relation(self, obj1, obj2, **hints):

        '''
        Description: Determines whether relationship is allowed between two objects
        '''
        # allow any relation:
        
        return True

class SubClientDBRouter:
    '''
    Description: Each client should have at least 3 databases: Raw datastore (RDS), Transactional datastore (TDS) and Analytical datastore (ADS). An apps data models should reflect this
                and the router uses this information to direct the queries to the relevant datastore.
    '''

    def db_for_read(self, model, **hints):

        try:
            if 'rds_' in model.__name__.lower():
                return str(request_cfg.cfg_rds)

            elif 'tds_' in model.__name__.lower():
                return str(request_cfg.cfg_tds)

            elif 'ads_' in model.__name__.lower():
                return str(request_cfg.cfg_ads)
        
        except AttributeError as AE:
            return None
        
        return None

    def db_for_write(self, model, **hints):

        try:
            if 'rds_' in model.__name__.lower() and hasattr(request_cfg, 'cfg_rds'):
                return str(request_cfg.cfg_rds)

            elif 'tds_' in model.__name__.lower() and hasattr(request_cfg, 'cfg_tds'):
                return str(request_cfg.cfg_tds)

            elif 'ads_' in model.__name__.lower() and hasattr(request_cfg, 'cfg_ads'):
                return str(request_cfg.cfg_ads)
        
        except AttributeError as AE:
            return None
        
        return None
    
    def allow_migrate(self, db, app_label=None, model_name=None, **hints):

        '''
        Description: Make sure the auth app does not appear in the RDS, TDS or ADS database
        '''

        # if the database nisn't client specific i.e. doesnt have 'RDS_', 'TDS_', 'ADS_' in the name, allow migrations to their natural database
        if db[0:4] not in SUB_CLIENT_DB_FLAGS:
            return True

        if "RDS_" in db:
            if model_name != None:
                if 'rds_' in model_name.lower():
                    return True

        elif "TDS_" in db:
            if 'tds_' in model_name.lower():
                return True

        elif "ADS_" in db:
            if 'ads_' in model_name.lower():
                return True

        if app_label == 'django_plotly_dash':
            return True
        
        
        return False
        


    # def allow_relation(self, obj1, obj2, **hints):

    #     '''
    #     Description: Determines whether relationship is allowed between two objects
    #     '''
    #     # allow any relation:
        
    #     return True
